@@ -1,8 +1,6 @@
 //
 // ODBCSQLServerTest.cpp
 //
-// $Id: //poco/Main/Data/ODBC/testsuite/src/ODBCSQLServerTest.cpp#5 $
-//
 // Copyright (c) 2006, Applied Informatics Software Engineering GmbH.
 // and Contributors.
 //
@@ -89,6 +87,7 @@ using Poco::DateTime;
 
 
 ODBCTest::SessionPtr ODBCSQLServerTest::_pSession;
+ODBCTest::SessionPtr ODBCSQLServerTest::_pEncSession;
 ODBCTest::ExecPtr    ODBCSQLServerTest::_pExecutor;
 std::string          ODBCSQLServerTest::_driver = MS_SQL_SERVER_ODBC_DRIVER;
 std::string          ODBCSQLServerTest::_dsn = MS_SQL_SERVER_DSN;
@@ -108,7 +107,7 @@ std::string ODBCSQLServerTest::_connectString = "DRIVER=" MS_SQL_SERVER_ODBC_DRI
 	;
 
 
-ODBCSQLServerTest::ODBCSQLServerTest(const std::string& name): 
+ODBCSQLServerTest::ODBCSQLServerTest(const std::string& name):
 	ODBCTest(name, _pSession, _pExecutor, _dsn, _uid, _pwd, _connectString)
 {
 }
@@ -129,13 +128,13 @@ void ODBCSQLServerTest::testBareboneODBC()
 		"Fifth FLOAT,"
 		"Sixth DATETIME)";
 
-	executor().bareboneODBCTest(dbConnString(), tableCreateString, 
+	executor().bareboneODBCTest(dbConnString(), tableCreateString,
 		SQLExecutor::PB_IMMEDIATE, SQLExecutor::DE_MANUAL, true, "CONVERT(VARBINARY(30),?)");
-	executor().bareboneODBCTest(dbConnString(), tableCreateString, 
+	executor().bareboneODBCTest(dbConnString(), tableCreateString,
 		SQLExecutor::PB_IMMEDIATE, SQLExecutor::DE_BOUND, true, "CONVERT(VARBINARY(30),?)");
-	executor().bareboneODBCTest(dbConnString(), tableCreateString, 
+	executor().bareboneODBCTest(dbConnString(), tableCreateString,
 		SQLExecutor::PB_AT_EXEC, SQLExecutor::DE_MANUAL, true, "CONVERT(VARBINARY(30),?)");
-	executor().bareboneODBCTest(dbConnString(), tableCreateString, 
+	executor().bareboneODBCTest(dbConnString(), tableCreateString,
 		SQLExecutor::PB_AT_EXEC, SQLExecutor::DE_BOUND, true, "CONVERT(VARBINARY(30),?)");
 
 	tableCreateString = "CREATE TABLE Test "
@@ -256,10 +255,10 @@ void ODBCSQLServerTest::testStoredProcedure()
 			"SET @outParam = -1; "
 			"END;"
 		, now;
-		
+
 		int i = 0;
 		session() << "{call storedProcedure(?)}", out(i), now;
-		assert(-1 == i);
+		assertTrue (-1 == i);
 		dropObject("PROCEDURE", "storedProcedure");
 
 		session() << "CREATE PROCEDURE storedProcedure(@inParam int, @outParam int OUTPUT) AS "
@@ -271,7 +270,7 @@ void ODBCSQLServerTest::testStoredProcedure()
 		i = 2;
 		int j = 0;
 		session() << "{call storedProcedure(?, ?)}", in(i), out(j), now;
-		assert(4 == j);
+		assertTrue (4 == j);
 		dropObject("PROCEDURE", "storedProcedure");
 
 		session() << "CREATE PROCEDURE storedProcedure(@ioParam int OUTPUT) AS "
@@ -282,7 +281,7 @@ void ODBCSQLServerTest::testStoredProcedure()
 
 		i = 2;
 		session() << "{call storedProcedure(?)}", io(i), now;
-		assert(4 == i);
+		assertTrue (4 == i);
 		dropObject("PROCEDURE", "storedProcedure");
 
 		session() << "CREATE PROCEDURE storedProcedure(@ioParam DATETIME OUTPUT) AS "
@@ -292,15 +291,15 @@ void ODBCSQLServerTest::testStoredProcedure()
 
 		DateTime dt(1965, 6, 18, 5, 35, 1);
 		session() << "{call storedProcedure(?)}", io(dt), now;
-		assert(19 == dt.day());
+		assertTrue (19 == dt.day());
 		dropObject("PROCEDURE", "storedProcedure");
 
 		k += 2;
 	}
 /*TODO - currently fails with following error:
 
-[Microsoft][ODBC SQL Server Driver][SQL Server]Invalid parameter 
-2 (''):  Data type 0x23 is a deprecated large object, or LOB, but is marked as output parameter.  
+[Microsoft][ODBC SQL Server Driver][SQL Server]Invalid parameter
+2 (''):  Data type 0x23 is a deprecated large object, or LOB, but is marked as output parameter.
 Deprecated types are not supported as output parameters.  Use current large object types instead.
 
 	session().setFeature("autoBind", true);
@@ -315,7 +314,7 @@ Deprecated types are not supported as output parameters.  Use current large obje
 	try{
 	session() << "{call storedProcedure(?, ?)}", in(inParam), out(outParam), now;
 	}catch(StatementException& ex){std::cout << ex.toString();}
-	assert(outParam == inParam);
+	assertTrue (outParam == inParam);
 	dropObject("PROCEDURE", "storedProcedure");
 	*/
 }
@@ -341,26 +340,26 @@ void ODBCSQLServerTest::testCursorStoredProcedure()
 			"BEGIN "
 			" SELECT * "
 			" FROM Person "
-			" WHERE Age < @ageLimit " 
+			" WHERE Age < @ageLimit "
 			" ORDER BY Age DESC; "
 			"END;"
 		, now;
 
 		people.clear();
 		int age = 13;
-		
+
 		session() << "{call storedCursorProcedure(?)}", in(age), into(people), now;
-		
-		assert (2 == people.size());
-		assert (Person("Simpson", "Bart", "Springfield", 12) == people[0]);
-		assert (Person("Simpson", "Lisa", "Springfield", 10) == people[1]);
+
+		assertTrue (2 == people.size());
+		assertTrue (Person("Simpson", "Bart", "Springfield", 12) == people[0]);
+		assertTrue (Person("Simpson", "Lisa", "Springfield", 10) == people[1]);
 
 		Statement stmt = ((session() << "{call storedCursorProcedure(?)}", in(age), now));
 		RecordSet rs(stmt);
-		assert (rs["LastName"] == "Simpson");
-		assert (rs["FirstName"] == "Bart");
-		assert (rs["Address"] == "Springfield");
-		assert (rs["Age"] == 12);
+		assertTrue (rs["LastName"] == "Simpson");
+		assertTrue (rs["FirstName"] == "Bart");
+		assertTrue (rs["Address"] == "Springfield");
+		assertTrue (rs["Age"] == 12);
 
 		dropObject("TABLE", "Person");
 		dropObject("PROCEDURE", "storedCursorProcedure");
@@ -387,7 +386,7 @@ void ODBCSQLServerTest::testStoredProcedureAny()
 		, now;
 
 		session() << "{call storedProcedure(?, ?)}", in(i), out(j), now;
-		assert(4 == AnyCast<int>(j));
+		assertTrue (4 == AnyCast<int>(j));
 		session() << "DROP PROCEDURE storedProcedure;", now;
 
 		session() << "CREATE PROCEDURE storedProcedure(@ioParam int OUTPUT) AS "
@@ -398,7 +397,7 @@ void ODBCSQLServerTest::testStoredProcedureAny()
 
 		i = 2;
 		session() << "{call storedProcedure(?)}", io(i), now;
-		assert(4 == AnyCast<int>(i));
+		assertTrue (4 == AnyCast<int>(i));
 		dropObject("PROCEDURE", "storedProcedure");
 
 		k += 2;
@@ -411,7 +410,7 @@ void ODBCSQLServerTest::testStoredProcedureDynamicAny()
 	for (int k = 0; k < 8;)
 	{
 		session().setFeature("autoBind", bindValue(k));
-		
+
 		DynamicAny i = 2;
 		DynamicAny j = 0;
 
@@ -422,7 +421,7 @@ void ODBCSQLServerTest::testStoredProcedureDynamicAny()
 		, now;
 
 		session() << "{call storedProcedure(?, ?)}", in(i), out(j), now;
-		assert(4 == j);
+		assertTrue (4 == j);
 		session() << "DROP PROCEDURE storedProcedure;", now;
 
 		session() << "CREATE PROCEDURE storedProcedure(@ioParam int OUTPUT) AS "
@@ -433,7 +432,7 @@ void ODBCSQLServerTest::testStoredProcedureDynamicAny()
 
 		i = 2;
 		session() << "{call storedProcedure(?)}", io(i), now;
-		assert(4 == i);
+		assertTrue (4 == i);
 		dropObject("PROCEDURE", "storedProcedure");
 
 		k += 2;
@@ -459,7 +458,7 @@ void ODBCSQLServerTest::testStoredFunction()
 
 		int i = 0;
 		session() << "{? = call storedFunction}", out(i), now;
-		assert(-1 == i);
+		assertTrue (-1 == i);
 		dropObject("PROCEDURE", "storedFunction");
 
 
@@ -472,7 +471,7 @@ void ODBCSQLServerTest::testStoredFunction()
 		i = 2;
 		int result = 0;
 		session() << "{? = call storedFunction(?)}", out(result), in(i), now;
-		assert(4 == result);
+		assertTrue (4 == result);
 		dropObject("PROCEDURE", "storedFunction");
 
 
@@ -487,8 +486,8 @@ void ODBCSQLServerTest::testStoredFunction()
 		int j = 0;
 		result = 0;
 		session() << "{? = call storedFunction(?, ?)}", out(result), in(i), out(j), now;
-		assert(4 == j);
-		assert(j == result);
+		assertTrue (4 == j);
+		assertTrue (j == result);
 		dropObject("PROCEDURE", "storedFunction");
 
 
@@ -506,18 +505,18 @@ void ODBCSQLServerTest::testStoredFunction()
 		j = 2;
 		result = 0;
 		session() << "{? = call storedFunction(?, ?)}", out(result), io(i), io(j), now;
-		assert(1 == j);
-		assert(2 == i);
-		assert(3 == result); 
+		assertTrue (1 == j);
+		assertTrue (2 == i);
+		assertTrue (3 == result);
 
 		Tuple<int, int> params(1, 2);
-		assert(1 == params.get<0>());
-		assert(2 == params.get<1>());
+		assertTrue (1 == params.get<0>());
+		assertTrue (2 == params.get<1>());
 		result = 0;
 		session() << "{? = call storedFunction(?, ?)}", out(result), io(params), now;
-		assert(1 == params.get<1>());
-		assert(2 == params.get<0>());
-		assert(3 == result); 
+		assertTrue (1 == params.get<1>());
+		assertTrue (2 == params.get<0>());
+		assertTrue (3 == result);
 
 		dropObject("PROCEDURE", "storedFunction");
 
@@ -614,6 +613,15 @@ void ODBCSQLServerTest::recreateFloatsTable()
 }
 
 
+void ODBCSQLServerTest::recreateUUIDsTable()
+{
+	dropObject("TABLE", "Strings");
+	try { session() << "CREATE TABLE Strings (str UNIQUEIDENTIFIER)", now; }
+	catch(ConnectionException& ce){ std::cout << ce.toString() << std::endl; fail ("recreateUUIDsTable()"); }
+	catch(StatementException& se){ std::cout << se.toString() << std::endl; fail ("recreateUUIDsTable()"); }
+}
+
+
 void ODBCSQLServerTest::recreateTuplesTable()
 {
 	dropObject("TABLE", "Tuples");
@@ -676,15 +684,15 @@ void ODBCSQLServerTest::recreateBoolTable()
 void ODBCSQLServerTest::recreateMiscTable()
 {
 	dropObject("TABLE", "MiscTest");
-	try 
-	{ 
+	try
+	{
 		session() << "CREATE TABLE MiscTest "
 			"(First VARCHAR(30),"
 			"Second VARBINARY(30),"
 			"Third INTEGER,"
 			"Fourth FLOAT,"
 			"Fifth DATETIME,"
-			"Sixth BIT)", now; 
+			"Sixth BIT)", now;
 	} catch(ConnectionException& ce){ std::cout << ce.toString() << std::endl; fail ("recreateMiscTable()"); }
 	catch(StatementException& se){ std::cout << se.toString() << std::endl; fail ("recreateMiscTable()"); }
 }
@@ -695,20 +703,20 @@ void ODBCSQLServerTest::recreateLogTable()
 	dropObject("TABLE", "T_POCO_LOG");
 	dropObject("TABLE", "T_POCO_LOG_ARCHIVE");
 
-	try 
-	{ 
+	try
+	{
 		std::string sql = "CREATE TABLE %s "
 			"(Source VARCHAR(max),"
 			"Name VARCHAR(max),"
 			"ProcessId INTEGER,"
 			"Thread VARCHAR(max), "
-			"ThreadId INTEGER," 
+			"ThreadId INTEGER,"
 			"Priority INTEGER,"
 			"Text VARCHAR(max),"
 			"DateTime DATETIME)";
 
-		session() << sql, "T_POCO_LOG", now; 
-		session() << sql, "T_POCO_LOG_ARCHIVE", now; 
+		session() << sql, "T_POCO_LOG", now;
+		session() << sql, "T_POCO_LOG_ARCHIVE", now;
 
 	} catch(ConnectionException& ce){ std::cout << ce.toString() << std::endl; fail ("recreateLogTable()"); }
 	catch(StatementException& se){ std::cout << se.toString() << std::endl; fail ("recreateLogTable()"); }
@@ -726,13 +734,28 @@ void ODBCSQLServerTest::recreateUnicodeTable()
 }
 
 
+void ODBCSQLServerTest::recreateEncodingTables()
+{
+#if defined (POCO_ODBC_UNICODE)
+	dropObject("TABLE", "Latin1Table");
+	try { session() << "CREATE TABLE Latin1Table (str NVARCHAR(30))", now; }
+	catch (ConnectionException& ce) { std::cout << ce.toString() << std::endl; fail("recreateEncodingTables()"); }
+	catch (StatementException& se) { std::cout << se.toString() << std::endl; fail("recreateEncodingTables()"); }
+#endif
+}
+
+
 CppUnit::Test* ODBCSQLServerTest::suite()
 {
 	if ((_pSession = init(_driver, _dsn, _uid, _pwd, _connectString, _db)))
 	{
 		std::cout << "*** Connected to [" << _driver << "] test database." << std::endl;
+		std::string enc = "Latin1";
+		if ((_pEncSession = init(_driver, _dsn, _uid, _pwd, _connectString, _db, enc)))
+			std::cout << "*** Connected to [" << _driver << "] test database, encoding: [" << enc << "]." << std::endl;
+		// ...
 
-		_pExecutor = new SQLExecutor(_driver + " SQL Executor", _pSession);
+		_pExecutor = new SQLExecutor(_driver + " SQL Executor", _pSession, _pEncSession);
 
 		CppUnit::TestSuite* pSuite = new CppUnit::TestSuite("ODBCSQLServerTest");
 
@@ -789,6 +812,7 @@ CppUnit::Test* ODBCSQLServerTest::suite()
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testDateTime);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testFloat);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testDouble);
+		CppUnit_addTest(pSuite, ODBCSQLServerTest, testUUID);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testTuple);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testTupleVector);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testStoredProcedure);
@@ -814,6 +838,7 @@ CppUnit::Test* ODBCSQLServerTest::suite()
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testTransactor);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testNullable);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testUnicode);
+		CppUnit_addTest(pSuite, ODBCSQLServerTest, testEncoding);
 		CppUnit_addTest(pSuite, ODBCSQLServerTest, testReconnect);
 
 		return pSuite;
